@@ -1,3 +1,4 @@
+/* $Id: CbcBranchDynamic.hpp 1271 2009-11-05 15:57:25Z forrest $ */
 // Copyright (C) 2005, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #ifndef CbcBranchDynamic_H
@@ -31,7 +32,7 @@ public:
 			      double downDynamicPseudoCost, double upDynamicPseudoCost);
   
   // Useful constructor - passed  model index and pseudo costs
-  CbcSimpleIntegerDynamicPseudoCost (CbcModel * model, int dummy, int iColumn, 
+  CbcSimpleIntegerDynamicPseudoCost (CbcModel * model,int dummy, int iColumn, 
 			      double downDynamicPseudoCost, double upDynamicPseudoCost);
   
   // Copy constructor 
@@ -46,26 +47,14 @@ public:
   // Destructor 
   virtual ~CbcSimpleIntegerDynamicPseudoCost ();
   
-  using CbcObject::infeasibility ;
   /// Infeasibility - large is 0.5
-  virtual double infeasibility(int & preferredWay) const;
+  virtual double infeasibility(const OsiBranchingInformation * info,
+			       int &preferredWay) const;
 
-  using CbcObject::createBranch ;
   /// Creates a branching object
-  virtual CbcBranchingObject * createBranch(int way) ;
-
-  /// Infeasibility - large is 0.5
-  virtual double infeasibility(const OsiSolverInterface * solver, 
-			       const OsiBranchingInformation * info, int & preferredWay) const;
+  virtual CbcBranchingObject * createCbcBranch(OsiSolverInterface * solver,const OsiBranchingInformation * info, int way) ;
 
 
-  /** Create a branching object and indicate which way to branch first.
-      
-      The branching object has to know how to create branches (fix
-      variables, etc.)
-  */
-  virtual CbcBranchingObject * createBranch(OsiSolverInterface * solver,
-					    const OsiBranchingInformation * info, int way) ;
   /// Fills in a created branching object
   void fillCreateBranch(CbcIntegerBranchingObject * branching, const OsiBranchingInformation * info, int way) ;
 
@@ -100,12 +89,29 @@ public:
   { return downDynamicPseudoCost_;}
   /// Set down pseudo cost
   void setDownDynamicPseudoCost(double value) ;
+  /// Modify down pseudo cost in a slightly different way
+  void updateDownDynamicPseudoCost(double value);
 
   /// Up pseudo cost
   inline double upDynamicPseudoCost() const
   { return upDynamicPseudoCost_;}
   /// Set up pseudo cost
   void setUpDynamicPseudoCost(double value);
+  /// Modify up pseudo cost in a slightly different way
+  void updateUpDynamicPseudoCost(double value);
+
+  /// Down pseudo shadow price cost
+  inline double downShadowPrice() const
+  { return downShadowPrice_;}
+  /// Set down pseudo shadow price cost
+  inline void setDownShadowPrice(double value) 
+  { downShadowPrice_ = value;}
+  /// Up pseudo shadow price cost
+  inline double upShadowPrice() const
+  { return upShadowPrice_;}
+  /// Set up pseudo shadow price cost
+  inline void setUpShadowPrice(double value)
+  { upShadowPrice_ = value;}
 
   /// Up down separator
   inline double upDownSeparator() const
@@ -122,7 +128,7 @@ public:
   { sumDownCost_=value;}
   /// Add to down sum cost and set last and square
   inline void addToSumDownCost(double value)
-  { sumDownCost_+=value;lastDownCost_=value;sumDownCostSquared_ += value*value;}
+  { sumDownCost_+=value;lastDownCost_=value;}
 
   /// Up sum cost
   inline double sumUpCost() const
@@ -132,7 +138,7 @@ public:
   { sumUpCost_=value;}
   /// Add to up sum cost and set last and square
   inline void addToSumUpCost(double value)
-  { sumUpCost_+=value;lastUpCost_=value;sumUpCostSquared_ += value*value;}
+  { sumUpCost_+=value;lastUpCost_=value;}
 
   /// Down sum change
   inline double sumDownChange() const
@@ -267,10 +273,10 @@ protected:
   double sumDownChange_;
   /// Sum of all changes to x when going up
   double sumUpChange_;
-  /// Sum down cost from strong or actual squared
-  mutable double sumDownCostSquared_;
-  /// Sum up cost from strong or actual squared
-  mutable double sumUpCostSquared_;
+  /// Current pseudo-shadow price estimate down
+  mutable double downShadowPrice_;
+  /// Current pseudo-shadow price estimate up
+  mutable double upShadowPrice_;
   /// Sum down decrease number infeasibilities from strong or actual
   double sumDownDecrease_;
   /// Sum up decrease number infeasibilities from strong or actual
@@ -304,10 +310,6 @@ protected:
   /// Number of times probing done 
   int numberTimesProbingTotal_;
   /// Number of times infeasible when tested
-#define CBC_INSTRUMENT
-#ifdef CBC_INSTRUMENT
-  mutable int numberTimesInfeasible_;
-#endif
   /** Method - 
       0 - pseudo costs
       1 - probing
