@@ -1,3 +1,4 @@
+/* $Id: CbcHeuristicRINS.cpp 1261 2009-10-30 12:45:20Z forrest $ */
 // Copyright (C) 2006, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #if defined(_MSC_VER)
@@ -24,9 +25,12 @@ CbcHeuristicRINS::CbcHeuristicRINS()
   numberSolutions_=0;
   numberSuccesses_=0;
   numberTries_=0;
+  stateOfFixing_ =0;
+  lastNode_=-999999;
   howOften_=100;
   decayFactor_ = 0.5; 
   used_=NULL;
+  whereFrom_=1+8+255*256;
 }
 
 // Constructor with model - assumed before cuts
@@ -37,12 +41,15 @@ CbcHeuristicRINS::CbcHeuristicRINS(CbcModel & model)
   numberSolutions_=0;
   numberSuccesses_=0;
   numberTries_=0;
+  stateOfFixing_ =0;
+  lastNode_=-999999;
   howOften_=100;
   decayFactor_ = 0.5; 
   assert(model.solver());
   int numberColumns = model.solver()->getNumCols();
   used_ = new char[numberColumns];
   memset(used_,0,numberColumns);
+  whereFrom_=1+8+255*256;
 }
 
 // Destructor 
@@ -66,9 +73,10 @@ CbcHeuristicRINS::operator=( const CbcHeuristicRINS& rhs)
     CbcHeuristic::operator=(rhs);
     numberSolutions_ = rhs.numberSolutions_;
     howOften_ = rhs.howOften_;
-    decayFactor_ = rhs.decayFactor_;
     numberSuccesses_ = rhs.numberSuccesses_;
     numberTries_ = rhs.numberTries_;
+    stateOfFixing_ = rhs.stateOfFixing_;
+    lastNode_=rhs.lastNode_;
     delete [] used_;
     if (model_&&rhs.used_) {
       int numberColumns = model_->solver()->getNumCols();
@@ -102,9 +110,10 @@ CbcHeuristicRINS::CbcHeuristicRINS(const CbcHeuristicRINS & rhs)
   CbcHeuristic(rhs),
   numberSolutions_(rhs.numberSolutions_),
   howOften_(rhs.howOften_),
-  decayFactor_(rhs.decayFactor_),
   numberSuccesses_(rhs.numberSuccesses_),
-  numberTries_(rhs.numberTries_)
+  numberTries_(rhs.numberTries_),
+  stateOfFixing_(rhs.stateOfFixing_),
+  lastNode_(rhs.lastNode_)
 {
   if (model_&&rhs.used_) {
     int numberColumns = model_->solver()->getNumCols();
@@ -116,10 +125,11 @@ CbcHeuristicRINS::CbcHeuristicRINS(const CbcHeuristicRINS & rhs)
 }
 // Resets stuff if model changes
 void 
-CbcHeuristicRINS::resetModel(CbcModel * model)
+CbcHeuristicRINS::resetModel(CbcModel * /*model*/)
 {
   //CbcHeuristic::resetModel(model);
   delete [] used_;
+  stateOfFixing_ = 0;
   if (model_&&used_) {
     int numberColumns = model_->solver()->getNumCols();
     used_ = new char[numberColumns];
@@ -169,16 +179,26 @@ CbcHeuristicRINS::solution(double & solutionValue,
       }
     }
   } 
-  if ((model_->getNodeCount()%howOften_)==0&&(model_->getCurrentPassNumber()==1||model_->getCurrentPassNumber()==999999)) {
+  int numberNodes=model_->getNodeCount();
+  if (howOften_==100) {
+    if (numberNodes<lastNode_+12)
+      return 0;
+    // Do at 50 and 100
+    if ((numberNodes>40&&numberNodes<=50)||(numberNodes>90&&numberNodes<100))
+      numberNodes=howOften_;
+  }
+  if ((numberNodes%howOften_)==0&&(model_->getCurrentPassNumber()==1||
+				   model_->getCurrentPassNumber()==999999)) {
+    lastNode_=model_->getNodeCount();
     OsiSolverInterface * solver = model_->solver();
 
     int numberIntegers = model_->numberIntegers();
     const int * integerVariable = model_->integerVariable();
   
     const double * currentSolution = solver->getColSolution();
-    OsiSolverInterface * newSolver = model_->continuousSolver()->clone();
-    //const double * colLower = newSolver->getColLower();
-    //const double * colUpper = newSolver->getColUpper();
+    OsiSolverInterface * newSolver = cloneBut(3); // was model_->continuousSolver()->clone();
+    int numberColumns=newSolver->getNumCols();
+    int numberContinuous = numberColumns-numberIntegers;
 
     double primalTolerance;
     solver->getDblParam(OsiPrimalTolerance,primalTolerance);
@@ -205,14 +225,78 @@ CbcHeuristicRINS::solution(double & solutionValue,
 	nFix++;
       }
     }
-    if (nFix>numberIntegers/5) {
+    int divisor=0;
+    if (5*nFix>numberIntegers) {
+      if (numberContinuous>2*numberIntegers&&nFix*10<numberColumns&&
+	  ((!numRuns_&&numberTries_>2)||stateOfFixing_)) {
+#define RINS_FIX_CONTINUOUS
+#ifdef RINS_FIX_CONTINUOUS
+	const double * colLower = newSolver->getColLower();
+	//const double * colUpper = newSolver->getColUpper();
+	int nAtLb=0;
+	//double sumDj=0.0;
+	const double * dj = newSolver->getReducedCost();
+	double direction=newSolver->getObjSense();
+	for (int iColumn=0;iColumn<numberColumns;iColumn++) {
+	  if (!newSolver->isInteger(iColumn)) {
+	    double value=bestSolution[iColumn];
+	    if (value<colLower[iColumn]+1.0e-8) {
+	      //double djValue = dj[iColumn]*direction;
+	      nAtLb++;
+	      //sumDj += djValue;
+	    }
+	  }
+	}
+	if (nAtLb) {
+	  // fix some continuous
+	  double * sort = new double[nAtLb];
+	  int * which = new int [nAtLb];
+	  //double threshold = CoinMax((0.01*sumDj)/static_cast<double>(nAtLb),1.0e-6);
+	  int nFix2=0;
+	  for (int iColumn=0;iColumn<numberColumns;iColumn++) {
+	    if (!newSolver->isInteger(iColumn)) {
+	      double value=bestSolution[iColumn];
+	      if (value<colLower[iColumn]+1.0e-8) {
+		double djValue = dj[iColumn]*direction;
+		if (djValue>1.0e-6) {
+		  sort[nFix2]=-djValue;
+		  which[nFix2++]=iColumn;
+		}
+	      }
+	    }
+	  }
+	  CoinSort_2(sort,sort+nFix2,which);
+	  divisor=4;
+	  if (stateOfFixing_>0)
+	    divisor=stateOfFixing_;
+	  else if (stateOfFixing_<-1)
+	    divisor=(-stateOfFixing_)-1;
+	  nFix2=CoinMin(nFix2,(numberColumns-nFix)/divisor);
+	  for (int i=0;i<nFix2;i++) {
+	    int iColumn = which[i];
+	    newSolver->setColUpper(iColumn,colLower[iColumn]);
+	  }
+	  delete [] sort;
+	  delete [] which;
+#ifdef CLP_INVESTIGATE2
+	  printf("%d integers have same value, and %d continuous fixed at lb\n",
+		 nFix,nFix2);
+#endif
+	}
+#endif
+      }
       //printf("%d integers have same value\n",nFix);
       returnCode = smallBranchAndBound(newSolver,numberNodes_,betterSolution,solutionValue,
                                          model_->getCutoff(),"CbcHeuristicRINS");
-      if (returnCode<0)
+      if (returnCode<0) {
 	returnCode=0; // returned on size
-      else
+	if (divisor)
+	  stateOfFixing_ = - divisor; // say failed
+      } else {
 	numRuns_++;
+	if (divisor)
+	  stateOfFixing_ =  divisor; // say small enough
+      }
       if ((returnCode&1)!=0)
 	numberSuccesses_++;
       //printf("return code %d",returnCode);
@@ -223,11 +307,11 @@ CbcHeuristicRINS::solution(double & solutionValue,
       } else {
 	//printf("\n");
       }
-      numberTries_++;
-      if ((numberTries_%10)==0&&numberSuccesses_*3<numberTries_)
-	howOften_ += static_cast<int> (howOften_*decayFactor_);
     }
 
+    numberTries_++;
+    if ((numberTries_%10)==0&&numberSuccesses_*3<numberTries_)
+      howOften_ += static_cast<int> (howOften_*decayFactor_);
     delete newSolver;
   }
   return returnCode;
@@ -248,6 +332,7 @@ CbcHeuristicRENS::CbcHeuristicRENS()
   :CbcHeuristic()
 {
   numberTries_=0;
+  whereFrom_=256+1;
 }
 
 // Constructor with model - assumed before cuts
@@ -256,6 +341,7 @@ CbcHeuristicRENS::CbcHeuristicRENS(CbcModel & model)
   :CbcHeuristic(model)
 {
   numberTries_=0;
+  whereFrom_=256+1;
 }
 
 // Destructor 
@@ -290,7 +376,7 @@ CbcHeuristicRENS::CbcHeuristicRENS(const CbcHeuristicRENS & rhs)
 }
 // Resets stuff if model changes
 void 
-CbcHeuristicRENS::resetModel(CbcModel * model)
+CbcHeuristicRENS::resetModel(CbcModel * )
 {
 }
 int
@@ -298,7 +384,8 @@ CbcHeuristicRENS::solution(double & solutionValue,
 			 double * betterSolution)
 {
   int returnCode=0;
-  if (numberTries_)
+  const double * bestSolution = model_->bestSolution();
+  if (numberTries_||(when()<2&&bestSolution))
     return 0; 
   numberTries_++;
   OsiSolverInterface * solver = model_->solver();
@@ -307,7 +394,7 @@ CbcHeuristicRENS::solution(double & solutionValue,
   const int * integerVariable = model_->integerVariable();
   
   const double * currentSolution = solver->getColSolution();
-  OsiSolverInterface * newSolver = model_->continuousSolver()->clone();
+  OsiSolverInterface * newSolver = cloneBut(3); // was model_->continuousSolver()->clone();
   const double * colLower = newSolver->getColLower();
   const double * colUpper = newSolver->getColUpper();
 
@@ -318,6 +405,8 @@ CbcHeuristicRENS::solution(double & solutionValue,
   int numberFixed=0;
   int numberTightened=0;
   int numberAtBound=0;
+  int numberColumns=newSolver->getNumCols();
+  int numberContinuous = numberColumns-numberIntegers;
 
   for (i=0;i<numberIntegers;i++) {
     int iColumn=integerVariable[i];
@@ -326,6 +415,8 @@ CbcHeuristicRENS::solution(double & solutionValue,
     double upper = colUpper[iColumn];
     value = CoinMax(value,lower);
     value = CoinMin(value,upper);
+#define RENS_FIX_ONLY_LOWER
+#ifndef RENS_FIX_ONLY_LOWER
     if (fabs(value-floor(value+0.5))<1.0e-8) {
       value = floor(value+0.5);
       if (value==lower||value==upper)
@@ -338,8 +429,84 @@ CbcHeuristicRENS::solution(double & solutionValue,
       newSolver->setColLower(iColumn,floor(value));
       newSolver->setColUpper(iColumn,ceil(value));
     }
+#else
+    if (fabs(value-floor(value+0.5))<1.0e-8&&
+	floor(value+0.5)==lower) {
+      value = floor(value+0.5);
+      numberAtBound++;
+      newSolver->setColLower(iColumn,value);
+      newSolver->setColUpper(iColumn,value);
+      numberFixed++;
+    } else if (colUpper[iColumn]-colLower[iColumn]>=2.0) {
+      numberTightened++;
+      if (fabs(value-floor(value+0.5))<1.0e-8) {
+	value = floor(value+0.5);
+	if (value<upper) {
+	  newSolver->setColLower(iColumn,CoinMax(value-1.0,lower));
+	  newSolver->setColUpper(iColumn,CoinMin(value+1.0,upper));
+	} else {
+	  newSolver->setColLower(iColumn,upper-1.0);
+	}
+      } else {
+	newSolver->setColLower(iColumn,floor(value));
+	newSolver->setColUpper(iColumn,ceil(value));
+      }
+    }
+#endif
   }
   if (numberFixed>numberIntegers/5) {
+      if (numberContinuous>numberIntegers&&numberFixed<numberColumns/5) {
+#define RENS_FIX_CONTINUOUS
+#ifdef RENS_FIX_CONTINUOUS
+	const double * colLower = newSolver->getColLower();
+	//const double * colUpper = newSolver->getColUpper();
+	int nAtLb=0;
+	double sumDj=0.0;
+	const double * dj = newSolver->getReducedCost();
+	double direction=newSolver->getObjSense();
+	for (int iColumn=0;iColumn<numberColumns;iColumn++) {
+	  if (!newSolver->isInteger(iColumn)) {
+	    double value=currentSolution[iColumn];
+	    if (value<colLower[iColumn]+1.0e-8) {
+	      double djValue = dj[iColumn]*direction;
+	      nAtLb++;
+	      sumDj += djValue;
+	    }
+	  }
+	}
+	if (nAtLb) {
+	  // fix some continuous
+	  double * sort = new double[nAtLb];
+	  int * which = new int [nAtLb];
+	  double threshold = CoinMax((0.01*sumDj)/static_cast<double>(nAtLb),1.0e-6);
+	  int nFix2=0;
+	  for (int iColumn=0;iColumn<numberColumns;iColumn++) {
+	    if (!newSolver->isInteger(iColumn)) {
+	      double value=currentSolution[iColumn];
+	      if (value<colLower[iColumn]+1.0e-8) {
+		double djValue = dj[iColumn]*direction;
+		if (djValue>threshold) {
+		  sort[nFix2]=-djValue;
+		  which[nFix2++]=iColumn;
+		}
+	      }
+	    }
+	  }
+	  CoinSort_2(sort,sort+nFix2,which);
+	  nFix2=CoinMin(nFix2,(numberColumns-numberFixed)/2);
+	  for (int i=0;i<nFix2;i++) {
+	    int iColumn = which[i];
+	    newSolver->setColUpper(iColumn,colLower[iColumn]);
+	  }
+	  delete [] sort;
+	  delete [] which;
+#ifdef CLP_INVESTIGATE2
+	  printf("%d integers fixed (%d tightened) (%d at bound), and %d continuous fixed at lb\n",
+		 numberFixed,numberTightened,numberAtBound,nFix2);
+#endif
+	}
+#endif
+      }
 #ifdef COIN_DEVELOP
     printf("%d integers fixed and %d tightened\n",numberFixed,numberTightened);
 #endif
@@ -426,7 +593,6 @@ CbcHeuristicDINS::operator=( const CbcHeuristicDINS& rhs)
     CbcHeuristic::operator=(rhs);
     numberSolutions_ = rhs.numberSolutions_;
     howOften_ = rhs.howOften_;
-    decayFactor_ = rhs.decayFactor_;
     numberSuccesses_ = rhs.numberSuccesses_;
     numberTries_ = rhs.numberTries_;
     for (int i=0;i<numberKeptSolutions_;i++)
@@ -460,6 +626,10 @@ CbcHeuristicDINS::generateCpp( FILE * fp)
     fprintf(fp,"3  heuristicDINS.setHowOften(%d);\n",howOften_);
   else
     fprintf(fp,"4  heuristicDINS.setHowOften(%d);\n",howOften_);
+  if (maximumKeepSolutions_!=other.maximumKeepSolutions_)
+    fprintf(fp,"3  heuristicDINS.setMaximumKeep(%d);\n",maximumKeepSolutions_);
+  else
+    fprintf(fp,"4  heuristicDINS.setMaximumKeep(%d);\n",maximumKeepSolutions_);
   fprintf(fp,"3  cbcModel->addHeuristic(&heuristicDINS);\n");
 }
 
@@ -469,7 +639,6 @@ CbcHeuristicDINS::CbcHeuristicDINS(const CbcHeuristicDINS & rhs)
   CbcHeuristic(rhs),
   numberSolutions_(rhs.numberSolutions_),
   howOften_(rhs.howOften_),
-  decayFactor_(rhs.decayFactor_),
   numberSuccesses_(rhs.numberSuccesses_),
   numberTries_(rhs.numberTries_),
   maximumKeepSolutions_(rhs.maximumKeepSolutions_),
@@ -488,7 +657,7 @@ CbcHeuristicDINS::CbcHeuristicDINS(const CbcHeuristicDINS & rhs)
 }
 // Resets stuff if model changes
 void 
-CbcHeuristicDINS::resetModel(CbcModel * model)
+CbcHeuristicDINS::resetModel(CbcModel * )
 {
   //CbcHeuristic::resetModel(model);
   for (int i=0;i<numberKeptSolutions_;i++)
@@ -559,7 +728,7 @@ CbcHeuristicDINS::solution(double & solutionValue,
     double cutoff = model_->getCutoff();
     while(status) {
       status=0;
-      OsiSolverInterface * newSolver = model_->continuousSolver()->clone();
+      OsiSolverInterface * newSolver = cloneBut(3); // was model_->continuousSolver()->clone();
       const double * colLower = solver->getColLower();
       const double * colUpper = solver->getColUpper();
       
@@ -756,4 +925,285 @@ void CbcHeuristicDINS::setModel(CbcModel * model)
   values_=NULL;
 }
 
+// Default Constructor
+CbcHeuristicVND::CbcHeuristicVND() 
+  :CbcHeuristic()
+{
+  numberSolutions_=0;
+  numberSuccesses_=0;
+  numberTries_=0;
+  lastNode_=-999999;
+  howOften_=100;
+  decayFactor_ = 0.5; 
+  baseSolution_=NULL;
+  whereFrom_=1+8+255*256;
+  stepSize_=0;
+  k_=0;
+  kmax_=0;
+  nDifferent_=0;
+}
+
+// Constructor with model - assumed before cuts
+
+CbcHeuristicVND::CbcHeuristicVND(CbcModel & model)
+  :CbcHeuristic(model)
+{
+  numberSolutions_=0;
+  numberSuccesses_=0;
+  numberTries_=0;
+  lastNode_=-999999;
+  howOften_=100;
+  decayFactor_ = 0.5; 
+  assert(model.solver());
+  int numberColumns = model.solver()->getNumCols();
+  baseSolution_ = new double [numberColumns];
+  memset(baseSolution_,0,numberColumns*sizeof(double));
+  whereFrom_=1+8+255*256;
+  stepSize_=0;
+  k_=0;
+  kmax_=0;
+  nDifferent_=0;
+}
+
+// Destructor 
+CbcHeuristicVND::~CbcHeuristicVND ()
+{
+  delete [] baseSolution_;
+}
+
+// Clone
+CbcHeuristic *
+CbcHeuristicVND::clone() const
+{
+  return new CbcHeuristicVND(*this);
+}
+
+// Assignment operator 
+CbcHeuristicVND & 
+CbcHeuristicVND::operator=( const CbcHeuristicVND& rhs)
+{
+  if (this!=&rhs) {
+    CbcHeuristic::operator=(rhs);
+    numberSolutions_ = rhs.numberSolutions_;
+    howOften_ = rhs.howOften_;
+    numberSuccesses_ = rhs.numberSuccesses_;
+    numberTries_ = rhs.numberTries_;
+    lastNode_=rhs.lastNode_;
+    delete [] baseSolution_;
+    if (model_&&rhs.baseSolution_) {
+      int numberColumns = model_->solver()->getNumCols();
+      baseSolution_ = new double [numberColumns];
+      memcpy(baseSolution_,rhs.baseSolution_,numberColumns*sizeof(double));
+    } else {
+      baseSolution_=NULL;
+    }
+    stepSize_=rhs.stepSize_;
+    k_=rhs.k_;
+    kmax_=rhs.kmax_;
+    nDifferent_=rhs.nDifferent_;
+  }
+  return *this;
+}
+
+// Create C++ lines to get to current state
+void 
+CbcHeuristicVND::generateCpp( FILE * fp) 
+{
+  CbcHeuristicVND other;
+  fprintf(fp,"0#include \"CbcHeuristicVND.hpp\"\n");
+  fprintf(fp,"3  CbcHeuristicVND heuristicVND(*cbcModel);\n");
+  CbcHeuristic::generateCpp(fp,"heuristicVND");
+  if (howOften_!=other.howOften_)
+    fprintf(fp,"3  heuristicVND.setHowOften(%d);\n",howOften_);
+  else
+    fprintf(fp,"4  heuristicVND.setHowOften(%d);\n",howOften_);
+  fprintf(fp,"3  cbcModel->addHeuristic(&heuristicVND);\n");
+}
+
+// Copy constructor 
+CbcHeuristicVND::CbcHeuristicVND(const CbcHeuristicVND & rhs)
+:
+  CbcHeuristic(rhs),
+  numberSolutions_(rhs.numberSolutions_),
+  howOften_(rhs.howOften_),
+  numberSuccesses_(rhs.numberSuccesses_),
+  numberTries_(rhs.numberTries_),
+  lastNode_(rhs.lastNode_)
+{
+  if (model_&&rhs.baseSolution_) {
+    int numberColumns = model_->solver()->getNumCols();
+    baseSolution_ = new double [numberColumns];
+    memcpy(baseSolution_,rhs.baseSolution_,numberColumns*sizeof(double));
+  } else {
+    baseSolution_=NULL;
+  }
+  stepSize_=rhs.stepSize_;
+  k_=rhs.k_;
+  kmax_=rhs.kmax_;
+  nDifferent_=rhs.nDifferent_;
+}
+// Resets stuff if model changes
+void 
+CbcHeuristicVND::resetModel(CbcModel * /*model*/)
+{
+  //CbcHeuristic::resetModel(model);
+  delete [] baseSolution_;
+  if (model_&&baseSolution_) {
+    int numberColumns = model_->solver()->getNumCols();
+    baseSolution_ = new double [numberColumns];
+    memset(baseSolution_,0,numberColumns*sizeof(double));
+  } else {
+    baseSolution_=NULL;
+  }
+}
+/*
+  First tries setting a variable to better value.  If feasible then
+  tries setting others.  If not feasible then tries swaps
+  Returns 1 if solution, 0 if not */
+int
+CbcHeuristicVND::solution(double & solutionValue,
+			 double * betterSolution)
+{
+  numCouldRun_++;
+  int returnCode=0;
+  const double * bestSolution = model_->bestSolution();
+  if (!bestSolution)
+    return 0; // No solution found yet
+  if (numberSolutions_<model_->getSolutionCount()) {
+    // new solution - add info
+    numberSolutions_=model_->getSolutionCount();
+
+    int numberIntegers = model_->numberIntegers();
+    const int * integerVariable = model_->integerVariable();
+  
+    int i;
+    for (i=0;i<numberIntegers;i++) {
+      int iColumn = integerVariable[i];
+      const OsiObject * object = model_->object(i);
+      // get original bounds
+      double originalLower;
+      double originalUpper;
+      getIntegerInformation( object,originalLower, originalUpper); 
+      double value=bestSolution[iColumn];
+      if (value<originalLower) {
+	value=originalLower;
+      } else if (value>originalUpper) {
+	value=originalUpper;
+      }
+    }
+  } 
+  int numberNodes=model_->getNodeCount();
+  if (howOften_==100) {
+    if (numberNodes<lastNode_+12)
+      return 0;
+    // Do at 50 and 100
+    if ((numberNodes>40&&numberNodes<=50)||(numberNodes>90&&numberNodes<100))
+      numberNodes=howOften_;
+  }
+  if ((numberNodes%howOften_)==0&&(model_->getCurrentPassNumber()==1||
+				   model_->getCurrentPassNumber()==999999)) {
+    lastNode_=model_->getNodeCount();
+    OsiSolverInterface * solver = model_->solver();
+
+    int numberIntegers = model_->numberIntegers();
+    const int * integerVariable = model_->integerVariable();
+  
+    const double * currentSolution = solver->getColSolution();
+    OsiSolverInterface * newSolver = cloneBut(3); // was model_->continuousSolver()->clone();
+    //const double * colLower = newSolver->getColLower();
+    //const double * colUpper = newSolver->getColUpper();
+
+    double primalTolerance;
+    solver->getDblParam(OsiPrimalTolerance,primalTolerance);
+
+    // Sort on distance
+    double * distance = new double [numberIntegers];
+    int * which = new int [numberIntegers];
+    
+    int i;
+    int nFix=0;
+    double tolerance = 10.0*primalTolerance;
+    for (i=0;i<numberIntegers;i++) {
+      int iColumn=integerVariable[i];
+      const OsiObject * object = model_->object(i);
+      // get original bounds
+      double originalLower;
+      double originalUpper;
+      getIntegerInformation( object,originalLower, originalUpper); 
+      double valueInt=bestSolution[iColumn];
+      if (valueInt<originalLower) {
+	valueInt=originalLower;
+      } else if (valueInt>originalUpper) {
+	valueInt=originalUpper;
+      }
+      baseSolution_[iColumn]=currentSolution[iColumn];
+      distance[i]=fabs(currentSolution[iColumn]-valueInt);
+      which[i]=i;
+      if (fabs(currentSolution[iColumn]-valueInt)<tolerance) 
+	nFix++;
+    }
+    CoinSort_2(distance,distance+numberIntegers,which);
+    nDifferent_=numberIntegers-nFix;
+    stepSize_=nDifferent_/10;
+    k_ = stepSize_;
+    //nFix = numberIntegers-stepSize_;
+    for (i=0;i<nFix;i++) {
+      int j=which[i];
+      int iColumn=integerVariable[j];
+      const OsiObject * object = model_->object(i);
+      // get original bounds
+      double originalLower;
+      double originalUpper;
+      getIntegerInformation( object,originalLower, originalUpper); 
+      double valueInt=bestSolution[iColumn];
+      if (valueInt<originalLower) {
+	valueInt=originalLower;
+      } else if (valueInt>originalUpper) {
+	valueInt=originalUpper;
+      }
+      double nearest=floor(valueInt+0.5);
+      newSolver->setColLower(iColumn,nearest);
+      newSolver->setColUpper(iColumn,nearest);
+    }
+    delete [] distance;
+    delete [] which;
+    if (nFix>numberIntegers/5) {
+      //printf("%d integers have samish value\n",nFix);
+      returnCode = smallBranchAndBound(newSolver,numberNodes_,betterSolution,solutionValue,
+                                         model_->getCutoff(),"CbcHeuristicVND");
+      if (returnCode<0)
+	returnCode=0; // returned on size
+      else
+	numRuns_++;
+      if ((returnCode&1)!=0)
+	numberSuccesses_++;
+      //printf("return code %d",returnCode);
+      if ((returnCode&2)!=0) {
+	// could add cut
+	returnCode &= ~2;
+	//printf("could add cut with %d elements (if all 0-1)\n",nFix);
+      } else {
+	//printf("\n");
+      }
+      numberTries_++;
+      if ((numberTries_%10)==0&&numberSuccesses_*3<numberTries_)
+	howOften_ += static_cast<int> (howOften_*decayFactor_);
+    }
+
+    delete newSolver;
+  }
+  return returnCode;
+}
+// update model
+void CbcHeuristicVND::setModel(CbcModel * model)
+{
+  model_ = model;
+  // Get a copy of original matrix
+  assert(model_->solver());
+  delete [] baseSolution_;
+  int numberColumns = model->solver()->getNumCols();
+  baseSolution_ = new double [numberColumns];
+  memset(baseSolution_,0,numberColumns*sizeof(double));
+}
+
   
